# This work is licensed under the GNU GPLv2 or later.
# See the COPYING file in the top-level directory.

import configparser


class _TreeinfoData():
    def __init__(self, filename, arch, family, variant, version):

        self.filename = filename
        self.arch = arch or ''
        self.family = family or ''
        self.variant = variant or ''
        self.version = version or ''

    def match(self, treeinfo):
        if bool(treeinfo.arch.match(self.arch)) and \
           bool(treeinfo.family.match(self.family)) and \
           bool(treeinfo.variant.match(self.variant)) and \
           bool(treeinfo.version.match(self.version)):
            return True

        return False


def get_treeinfodata(filepath):
    arch = None
    family = None
    variant = None
    version = None

    config = configparser.ConfigParser()
    with open(filepath, 'r') as out:
        config.read(filepath)
        arch = ''
        family = ''
        variant = ''
        version = ''

        if 'arch' in config['general']:
            arch = config['general']['arch']

        if 'family' in config['general']:
            family = config['general']['family']

        if 'variant' in config['general']:
            variant = config['general']['variant']

        if 'version' in config['general']:
            version = config['general']['version']

    return _TreeinfoData(filepath, arch, family, variant, version)



# This work is licensed under the GNU GPLv2 or later.
# See the COPYING file in the top-level directory.

import glob
import logging
import os
import pytest

from . import util
from . import treeinfodata


def _get_treeinfodatapaths():
    """
    Collect treeinfo data and return a list of tuples:
        (osname, treeinfodatapaths)
    """
    treeinfodata_path = os.path.join(
        os.path.dirname(os.path.realpath(__file__)),
        'treeinfodata')

    ret = []
    allpaths = glob.glob(os.path.join(treeinfodata_path, "*", "*"))
    for osdir in sorted(allpaths, key=util.human_sort):
        osname = os.path.basename(osdir)
        # The .treeinfo may be under the following directories schema:
        # - distro/shortid/arch
        # - distro/shortid/variant/arch/
        # That's the reason we have to take check for the .treeinfo file recursively.
        treeinfodatapaths = glob.glob(os.path.join(osdir, "**", ".treeinfo"), recursive=True)
        if len(treeinfodatapaths):
            ret.append((osname, treeinfodatapaths))
    return ret


@pytest.mark.parametrize("testdata", _get_treeinfodatapaths(), ids=lambda d: d[0])
def test_treeinfo_detection(testdata):
    osname, treeinfodatapaths = testdata
    for treeinfodatapath in treeinfodatapaths:
        detected = []
        data = treeinfodata.get_treeinfodata(treeinfodatapath)

        for osxml2 in util.DataFiles.oses():
            for tree in osxml2.trees:
                if tree.treeinfo is None:
                    continue

                if data.match(tree.treeinfo):
                    if osname != osxml2.shortid:
                        logging.warning(
                            'treeinfo \'%s\' was matched by OS \'%s\' while '
                            'it should only be matched by OS \'%s\'',
                            data.filename, osxml2.shortid, osname)
                    else:
                        logging.info('treeinfo \'%s\' matched by OS \'%s\'',
                                     data.filename, osxml2.shortid)

                    if osxml2.shortid not in detected:
                        detected.append(osxml2.shortid)

        if detected == [osname]:
            continue

        raise AssertionError("data: %s\nMatched=%s but expected=%s" %
                (treeinfodatapath, detected, [osname]))



# This work is licensed under the GNU GPLv2 or later.
# See the COPYING file in the top-level directory.

from . import util


@util.os_parametrize('osxml', filter_installscripts=True)
def test_install_scripts(osxml):
    for script in osxml.installscripts:
        assert util.DataFiles.installscripts()[script]



# This work is licensed under the GNU GPLv2 or later.
# See the COPYING file in the top-level directory.

import enum
import http
import logging

import pytest
import requests

from . import util


class UrlType(enum.Enum):
    URL_GENERIC = 1
    URL_ISO = 2


iso_content_types = {
    # proper ISO mimetype
    'application/x-iso9660-image',
    # generic data
    'application/octet-stream',
    'binary/octet-stream',
    # ISO files on archive.netbsd.org
    'text/plain',
}


def _check_url(url, url_type):
    logging.info("url: %s, type: %s", url, url_type)
    headers = {'user-agent': 'Wget/1.0'}
    response = requests.head(url, allow_redirects=True, headers=headers)
    content_type = response.headers.get('content-type')
    if content_type:
        try:
            content_type = content_type[0:content_type.index(';')]
        except ValueError:
            pass
    logging.info("response: %s; code: %d; content-type: %s",
                 http.client.responses[response.status_code],
                 response.status_code, content_type)
    if not response.ok:
        return False
    if url_type == UrlType.URL_ISO:
        return content_type in iso_content_types
    return True


def _collect_os_urls():
    """
    Iterate the OS list and return a list of pairs (shortid, [url list)
    """
    ret = []

    for osxml in util.DataFiles.oses():
        urls = []
        urls.extend([(i.url, UrlType.URL_GENERIC) for i in osxml.images if i.url])
        urls.extend([(m.url, UrlType.URL_ISO) for m in osxml.medias if m.url])
        for t in osxml.trees:
            if not t.url:
                continue
            urls.append((t.url, UrlType.URL_GENERIC))
            url = t.url
            if not url.endswith('/'):
                url += '/'
            if t.kernel:
                urls.append((url + t.kernel, UrlType.URL_GENERIC))
            if t.initrd:
                urls.append((url + t.initrd, UrlType.URL_GENERIC))
        if urls:
            ret.append((osxml.shortid, urls))

    return ret


@pytest.mark.parametrize('testdata', _collect_os_urls(),
        ids=lambda testdata: testdata[0])
def test_urls(testdata):
    urls = testdata[1]
    broken = []
    for (url, url_type) in urls:
        # As some distro URLs are flaky, let's give it a try 3 times
        # before actually failing.
        for i in range(3):
            ok = _check_url(url, url_type)
            if ok:
                break

        if not ok:
            broken.append(url)
    assert broken == []
